# -*- coding: utf-8 -*-

"""
Unofficial Waifulabs API Wrapper

A Waifulabs wrapper to generate Waifu's with AI.

:copyright: (c) 2021 TaromaruYuki
:license: MIT, see LICENSE for more details.
"""

__title__ = "Waifulabs"
__author__ = "TaromaruYuki"
__license__ = "MIT"
__copyright__ = "Copyright 2021 TaromaruYuki"
__version__ = "1.1.0"

from .asynchronous import *
from .exception import *
from .sync import *
from .waifu import *
